<?php
attachModule(GUI, 'compinit');
attachModule(STR,'WebFunctions');

class Webpage {
    /*
     * Clase Webpage
     *
     * Esta clase contiene todos los elementos que hacen a una página web.
     *
     * NOTA: El div que agrupa todo tiene como nombre maincomponent. Ningún otro
     * componente puede tener ese nombre, ya que de lo contrario pueden aparecer
     * valores de un componente provenientes de su homónimo.
     *
     * NOTA: Para pasar información de un estilo a un componente, se debe generar
     * un array con las siguientes características:
     *      array( 'NOMBRECOMPONENTE-style' , array('NOMBREESTILO'=>'VALOR', ... ))
    */

    //Revisar esto, cambiaron cosas de los estilos.

    /* Declaración de variables */

    // Title: ID en la base de datos de idiomas que correspone al título de la página.
    public $title;
    // Stylesheets: Array con los estilos que se aplican a la página general.
    private $Stylesheets = array();

    // Variables de los componentes internos.
    private $GLOBAL_VARIABLES = array();
    private $PREPROCESSOR = null;
    private $JAVASCRIPTS = array();
    private $COMPONENTS = array();

    public function __construct($title = NULL) {
        $this->title = $title;
    }
    public function addStylesheet($name, $type) {
        $this->Stylesheets[$name] = new Stylesheet($name, $type);
    }
    public function addAttribute($name,$attribute,$value) {
        /*
         * Esta función agrega atributos al stylesheet indicado en el nombre.
        */
        $this->Stylesheets[$name]->addAttribute($attribute, $value);
    }
    public function addComponent($Component) {
        /*
         * Función addComponent
         *
         * Esta función agrega componentes a la página principal.
        */
        $this->COMPONENTS[] = $Component;
    }
    public function addPreprocessor($PREPROCESSOR) {
        /*
     * Función addPreprocessor
     *
     * Esta función agrega el código de preprocesamiento que ejecuta la página.
        */

        // Para direccionar a un archivo externo, escribir "File: Nombre_Del_Archivo.php"
        $this->PREPROCESSOR = $PREPROCESSOR;
    }
    public function retrieveStyles() {
        /*
         * Función retrieveStyles
         *
         * Esta función invoca en todos los componentes contenidos la función
         * retrieveStyles, a fin de escribir todos los estilos en el head de la página.
         * Cada componente, previo a la devolución de las stylesheets, actualiza su
         * propio stylesheet, actualizando sus valores por aquellos indicados en
         * las variables globales.
        */

        // Creo el array con todos los estilos a ser devueltos
        $styleinfo = array();
        // Le pido a cada componente que devuelva sus estilos internos.
        foreach($this->COMPONENTS as $Component) {
            // Extraigo los estilos devueltos.
            $aux = $Component->retrieveStyles();
            // Verifico si me devolvieron un arreglo de estilos, o un estilo único.
            if(is_array($aux)) {
                // Si me devolvieron un arreglo de estilos, agrego los estilos individuales a $styleinfo
                foreach ($aux as $style) {
                    // Antes de agregar el estilo, verifico que no sea nulo.
                    if(!is_null($style)) {
                        $styleinfo[] = $style;
                    }
                }
            }else {
                // Si lo devuelvo no es un arreglo, verifico que sea por lo menos una hoja de estilos.
                if(get_class($aux)=='Stylesheet') {
                    // Si aux es una Stylesheet, la agrego.
                    $styleinfo[] = $aux;
                }
            }
        }
        // Agrego los estilos de la página.
        foreach ($this->Stylesheets as $style) {
            $styleinfo[] = $style;
        }
        // Devuelvo los estilos
        return $styleinfo;
    }
    public function preprocess() {
        /*
         * Función preprocess.
         *
         * Esta función ejecuta un fragmento de código definido por el usuario,
         * el cual procesa las variables devueltas por una forma, o extrae datos
         * de alguna base de datos.
        */

        // Primero extraigo las variables del form manager
        $this->GLOBAL_VARIABLES = array_merge($_GET,$_POST);

        // Faltarían aqui algunas comprobaciones de seguridad.

        // Verifico si se dieron instrucciones de preprocesamiento
        if($this->PREPROCESSOR!=null) {
            // Verifico si el preprocesador es un archivo en este mismo directorio.
            if (StartsWith($this->PREPROCESSOR, 'File: ')) {
                // Si el codigo de preprocesado es un archivo PHP, extraigo primero el nombre del archivo.
                $phpfile = substr($this->PREPROCESSOR, strlen('File: '));
                // Genero el nombre de la instrucción del
                $instruction = substr($phpfile,0,-4);
                // Genero el path e incluyo el archivo
                $phpfile = 'AECore/config/preprocess/'.$phpfile;
                if (file_exists($phpfile)) {
                    include_once $phpfile;
                    // Ejecuto la funcion asociada al nombres del archivo. (Ej Archivo.php -> Archivo();
                    call_user_func($instruction, &$this->GLOBAL_VARIABLES);
                }else {
                    // Si no se encuentra el archivo, generar el panic correspondiente.

                }
            }else {
                // Si el AECode esta incluido, lo ejecuto.
                $AERunner = new AECodeRunner($this->GLOBAL_VARIABLES);
                $AERunner->processAECODE($this->PREPROCESSOR);
            }
        }
        // Le doy a cada componente las variables procesadas, para que las use.
        foreach($this->COMPONENTS as $Component) {
            $Component->attachGlobalVariables($this->GLOBAL_VARIABLES);
        }
        // Actualizo los estilos de la página
        if (array_key_exists('maincomponent-style', $this->GLOBAL_VARIABLES)) {
            // Extraigo el arreglo conteniendo la información del estilo

            /*
             * Formato del arreglo maincomponent-style:
             *
             * Este es un arreglo de arreglos. Cada indice corresponde a un estilo a modificarse.
             *
             * maincomponent=[[name,attribute,value],[name,attribute,value],[name,attribute,value] ... ]
             *
            */
            $styleinfo = $this->GLOBAL_VARIABLES['maincomponent-style'];
            // Verifico que la información que obtuve sea un arreglo.
            if(is_array($styleinfo)) {
                // Loopeo por los estilos para modificar cada uno individualmente.
                foreach($styleinfo as $style) {
                    // Escribo y sobreescribo los nuevos datos del estilo.
                    $this->Stylesheets[$style['name']]->addAttribute($style['attribute'], $style['value']);
                }
            }else {
                // Si no lo es, genero un panic invisible para reportar el error.
                // Intento ver que dato fue enviado para mejorar el debuggeo.
                panic2('Webpage', 2000, 'Information for skin was: '.print_r($styleinfo, true));
            }
        }


    }
    public function HTMLize() {
        /*
         * Función HTMLize
         *
         * Esta función invoca a todos los objetos contenidos a devolver su código html,
         * verificando previamente para el caso del maincomponent si se le pasó información
         * acerca del aspecto visual desde las variables globales.
        */

        // Instancio el OutputManager
        $output = new OutputManager();
        // Genero los componentes internos.
        foreach($this->COMPONENTS as $Component) {
            $output->addOutput($this->ExecuteComponent($Component));
        }
        // Devuelvo el output generado
        return $output->export_text();
    }
    private function ExecuteComponent($Component) {
        /*
         * Función ExecuteComponent
         *
         * Esta función analiza la ejecución condicional de un componente y
         * decide si debe construir su código HTML o no. La versión estándar
         * está pensada para el FrameWidget, llamando a cada componente interno
         * y pidiéndole su código HTML. Previo a la generación de código, se evalúa
         * si la ejecución es condicional. De serlo, se evalúa primero si el tipo
         * de ejecución es IF o WHILE, para luego ejecutar el código correspondiente,
         * verificando que la condición de error siempre se cumpla.
        */

        // Verifico primero si hay ejecución condicional sobre este elemento.
        if ($Component->CONDITONAL_EXECUTION == NULL) {
            // Como no hay ejecución condicional fijada, genero la salida como siempre.
            return $Component->generateHTML();
        }else {
            // Como hay ejecución condicional, determino primero el tipo de ejecución a realizarse.
            $type = $Component->CONDITONAL_EXECUTION['type'];
            switch ($type) {
                case 'IF':
                // Evaluo primero la truecondition.
                    $AECODE = new AECodeRunner($Component->GLOBAL_VARIABLES);
                    $truecondition = $AECODE->executeAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                    // Reviso que la truecondition sea un valor boolean;
                    if(is_bool($truecondition)) {
                        // Si truecondition es boolean, lo evalúo.
                        if($truecondition) {
                            // Si truecondition se cumple, genero el HTML y lo devuelvo.
                            return $Component->generateHTML();
                        }
                    }else {
                        // Si no es boolean, reporto el error.
                        panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    }
                    break;
                case 'WHILE':
                // Evaluo primero la truecondition.
                    $AECODE = new AECodeRunner($Component->GLOBAL_VARIABLES);
                    $truecondition = $AECODE->executeAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                    // Reviso que la truecondition sea un valor boolean;
                    if(is_bool($truecondition)) {
                        // Si truecondition es boolean, lo evalúo.
                        // Genero la variable de salida output
                        $output = new OutputManager();
                        // Evaluo el while, verificando también el máximo
                        while ($truecondition && $Component->CONDITONAL_EXECUTION['maximum']>0) {
                            // Si es true, agrego el contenido del objeto.
                            $output->addOutput($Component->generateHTML());
                            // Si hay procesamiento adicional pedido, lo realizo.
                            if($Component->CONDITONAL_EXECUTION['extraprocessing']!=null) {
                                eval($Component->CONDITONAL_EXECUTION['extraprocessing']);
                                // No es necesario copiar las variables globales al resto de los componentes, ya que
                                // el manejo se realiza por referencia, no por valor.
                            }
                            // Decremento maximum en una unidad.
                            $Component->CONDITONAL_EXECUTION['maximum']--;
                            // Evaluo la nueva truecondition
                            $truecondition = $AECODE->executeAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                            // Si no es boolean, termino la ejecución del while, reportando el error.
                            if(!is_bool($truecondition)) {
                                panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                                break;
                            }
                            // Vuelve a empezar la ejecución del While
                        }
                        // Finalizado el while, devuelvo el código a quien llamó a la función.
                        return $output->export();
                    }else {
                        // Si no es boolean, reporto el error.
                        panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    }
                    break;
                default:
                // Si esto se ejecuta, es que la ejecución condicional no pudo reconocer el tipo de ejecución
                    panic2(get_class($Component).': '.$Component->ID, 2002, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    // Por motivos de estética, no se va a generar el objeto si las condiciones de ejecución no están bien formuladas.
                    break;
            }

        }


    }
}
?>
