<?php
attachModule(GUI, 'compinit');
attachModule(STR,'WebFunctions');

class Widget {
    /*
     * Clase Widget
     * 
     * Esta clase es la clase madre de los demás widgets. Contiene ya el código para
     * agregar y modificar los estilos, así como permitir la ejecución condicional de código.
    */

    public $ID;
    public $Styles;
    public $Components = array();
    public $GLOBAL_VARIABLES = array();
    public $CONDITONAL_EXECUTION = null;

    public function  __construct($name) {
        /*
         * Esta función construye lo básico de cualquier objeto widget.
         * La construcción específica de cada objeto está en su propia clase.
        */
        $this->ID = $name;
        $this->Styles = new Stylesheet($name,TYPE_ID);
    }
    public function addAttribute($name,$value) {
        /*
         * Esta función agrega atributos al stylesheet del widget.
        */
        $this->Styles->addAttribute($name,$value);
    }
    public function addComponent($Component) {
        /*
         * Función addComponent
         *
         * Esta función agrega componentes a la página principal.
        */
        $this->Components[] = $Component;
    }
    public function setConditionalExecution($type,$truecondicion,$extraprocessing=null,$maximum=20) {
        /*
         * Esta función fija los parámetros de evaluación condicional. El valor maximum se fija
         * como un límite para las ejecuciones WHILE, para evitar que la página quede en un bucle infinito.
         * El límite máximo se puede modificar en caso de ser necesario.
         *
         * Formato de los datos:
         * type: IF o WHILE
         * truecondicion: [#AECODE|CONDEXEC var1==var2|AECODE#]
         * extraprocessing: codigo aecode valido
         * maximum: Integer
        */
        $this->CONDITONAL_EXECUTION = array('type' => $type,'truecondition' => $truecondicion,'extraprocessing' => $extraprocessing,'maximum' => $maximum);
    }
    public function attachGlobalVariables(&$globalvariables) {
        /*
         * attachGlobalVariables
         *
         * Esta función recibe como parámetro un array conteniendo las variables
         * globales definidas, asignándolas al propio Widget, y transmitiendo esa
         * información a todos los demás componentes que pueda tener dentro.
        */
        // Guardo el arreglo dentro del objeto.
        $this->GLOBAL_VARIABLES = $globalvariables;
        // Transmito la información a todos los objetos contenidos dentro.
        foreach($this->Components as $Component) {
            // Como todos los objetos extienden la clase Widget, puedo
            // llamar a la misma función attachGlobalVariables
            $Component->attachGlobalVariables($this->GLOBAL_VARIABLES);
        }
    }
    public function retrieveStyles() {
        /*
         * Función retrieveStyles
         *
         * Esta función invoca en todos los componentes contenidos la función
         * retrieveStyles, a fin de escribir todos los estilos en el head de la página.
         * Cada componente, previo a la devolución de las stylesheets, actualiza su
         * propio stylesheet, actualizando sus valores por aquellos indicados en
         * las variables globales.
        */
        // Primero que nada, verifico si debo hacer cambios al estilo propio.
        if (array_key_exists($this->ID.'-style', $this->GLOBAL_VARIABLES)) {
            // Extraigo el arreglo conteniendo la información del estilo

            /*
             * Formato del arreglo maincomponent-style:
             *
             * Este es un arreglo de arreglos. Cada indice corresponde a un estilo a modificarse.
             *
             * maincomponent=[[name,attribute,value],[name,attribute,value],[name,attribute,value] ... ]
             *
            */
            $styleinfo = $this->GLOBAL_VARIABLES[$this->ID.'-style'];
            // Verifico que la información que obtuve sea un arreglo.
            if(is_array($styleinfo)) {
                // Loopeo por los estilos para modificar cada uno individualmente.
                foreach($styleinfo as $style) {
                    // Escribo y sobreescribo los nuevos datos del estilo.
                    $this->Stylesheets[$style['name']]->addAttribute($style['attribute'], $style['value']);
                }
            }else {
                // Si no lo es, genero un panic invisible para reportar el error.
                // En el panic reporto el nombre de la clase, su ID, y la información
                // que fue suministrada en el arreglo.
                panic2(get_class($this).': '.$this->ID, 2001, 'Information for skin was: '.print_r($styleinfo, true));
            }
        }
        // Creo el array con todos los estilos a ser devueltos
        $styleinfo = array();
        // Le pido a cada componente que devuelva sus estilos internos.
        foreach($this->Components as $Component) {
            // Extraigo los estilos devueltos.
            $aux = $Component->retrieveStyles();
            // Verifico si me devolvieron un arreglo de estilos, o un estilo único.
            if(is_array($aux)) {
                // Si me devolvieron un arreglo de estilos, agrego los estilos individuales a $styleinfo
                foreach ($aux as $style) {
                    $styleinfo[] = $style;
                }
            }else {
                // Si lo devuelvo no es un arreglo, verifico que sea por lo menos una hoja de estilos.
                if(get_class($aux)=='Stylesheet') {
                    // Si aux es una Stylesheet, la agrego.
                    $styleinfo[] = $aux;
                }
            }
        }
        // Procesados todos los estilos de los componentes internos, agrego a $styleinfo el estilo propio.
        // Si los estilos del widget son un array, agrego cada estilo individual.
        if(is_array($this->Styles)) {
            foreach ($this->Styles as $style) {
                $styleinfo[] = $style;
            }
        }else {
            $styleinfo[] = $this->Styles;
        }
        // Devuelvo el array con todos los estilos.
        return $styleinfo;
    }
    public function generateHTML() {
        /*
         *  Función generateHTML
         *
         * Esta función genera el código HTML correspondiente al widget, sin
         * considerar cuestiones de ejecución condicional. Esta función es llamada
         * por HTMLize para ejecutarse en caso de estar verificada la ejecución condicional.
        */

        // Genero la variable de salida output
        $output = new OutputManager();
        // Agrego el tag div correspondiente al framewidget.
        $output->addOutput(insertTag('div', true, false, false, array('id'=>$this->ID), true));
        // Agrego a output la salida en HTML de cada componente.
        foreach($this->Components as $Component) {
            $output->addOutput($this->ExecuteComponent($Component));
        }
        // Cierro el tag div correspondiente al framewidget
        $output->addOutput(insertTag('div', false, false, false, null, true));
        // Devuelvo output a quien llamó la función.
        return $output->export_text();
    }
    public function ExecuteComponent($Component) {
        /*
         * Función ExecuteComponent
         *
         * Esta función analiza la ejecución condicional de un componente y
         * decide si debe construir su código HTML o no. La versión estándar
         * está pensada para el FrameWidget, llamando a cada componente interno
         * y pidiéndole su código HTML. Previo a la generación de código, se evalúa
         * si la ejecución es condicional. De serlo, se evalúa primero si el tipo
         * de ejecución es IF o WHILE, para luego ejecutar el código correspondiente,
         * verificando que la condición de error siempre se cumpla.
        */

        // Verifico primero si hay ejecución condicional sobre este elemento.
        if ($Component->CONDITONAL_EXECUTION == NULL) {
            // Como no hay ejecución condicional fijada, genero la salida como siempre.
            return $Component->generateHTML();
        }else {
            // Como hay ejecución condicional, determino primero el tipo de ejecución a realizarse.
            $type = $Component->CONDITONAL_EXECUTION['type'];
            switch ($type) {
                case 'IF':
                // Evaluo primero la truecondition.
                    $AECODE = new AECodeRunner($Component->GLOBAL_VARIABLES);
                    $truecondition = $AECODE->processAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                    // Reviso que la truecondition sea un valor boolean;
                    if(is_bool($truecondition)) {
                        // Si truecondition es boolean, lo evalúo.
                        if($truecondition) {
                            // Si truecondition se cumple, genero el HTML y lo devuelvo.
                            return $Component->generateHTML();
                        }
                    }else {
                        // Si no es boolean, reporto el error.
                        panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    }
                    break;
                case 'WHILE':
                // Evaluo primero la truecondition.
                    $AECODE = new AECodeRunner($Component->GLOBAL_VARIABLES);
                    $truecondition = $AECODE->processAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                    // Reviso que la truecondition sea un valor boolean;
                    if(is_bool($truecondition)) {
                        // Si truecondition es boolean, lo evalúo.
                        // Genero la variable de salida output
                        $output = new OutputManager();
                        // Evaluo el while, verificando también el máximo
                        while ($truecondition && $Component->CONDITONAL_EXECUTION['maximum']>0) {
                            // Si es true, agrego el contenido del objeto.
                            $output->addOutput($Component->generateHTML());
                            // Si hay procesamiento adicional pedido, lo realizo.
                            if($Component->CONDITONAL_EXECUTION['extraprocessing']!=null) {
                                $AECODE->processAECODE($Component->CONDITONAL_EXECUTION['extraprocessing']);
                                // No es necesario copiar las variables globales al resto de los componentes, ya que
                                // el manejo se realiza por referencia, no por valor.
                            }
                            // Decremento maximum en una unidad.
                            $Component->CONDITONAL_EXECUTION['maximum']--;
                            // Evaluo la nueva truecondition
                            $truecondition = $AECODE->executeAECODE($Component->CONDITONAL_EXECUTION['truecondition']);
                            // Si no es boolean, termino la ejecución del while, reportando el error.
                            if(!is_bool($truecondition)) {
                                panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                                break;
                            }
                            // Vuelve a empezar la ejecución del While
                        }
                        // Finalizado el while, devuelvo el código a quien llamó a la función.
                        return $output->export();
                    }else {
                        // Si no es boolean, reporto el error.
                        panic2(get_class($Component).': '.$Component->ID, 2003, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    }
                    break;
                default:
                // Si esto se ejecuta, es que la ejecución condicional no pudo reconocer el tipo de ejecución
                    panic2(get_class($Component).': '.$Component->ID, 2002, 'Conditional execution information: '.print_r($Component->CONDITONAL_EXECUTION, true));
                    // Por motivos de estética, no se va a generar el objeto si las condiciones de ejecución no están bien formuladas.
                    break;
            }

        }


    }
}
?>
